# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url

from . import views

urlpatterns = patterns("",
    url(r"^login/(?P<uuid>[\da-f]{8}-([\da-f]{4}-){3}[\da-f]{12})/$", views.LOTLogin.as_view(), name="login"),
)



from uuid import uuid4

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except:
    from django.apps import apps
    user_app, user_model = settings.AUTH_USER_MODEL.split('.')
    User = apps.get_app_config(user_app).get_model(user_model)

from django.utils.timezone import now


LOT_SETTINGS = getattr(settings, 'LOT', {
    'fast-login': {
        'name': _(u'Fast login'),
        'duration': 60,
        'one-time': True,
    },
    'slow-login': {
        'name': _(u'Slow login'),
        'duration': 60*60*24,
        'one-time': False
    },
    'always-login': {
        'name': _(u'Always login'),
        'one-time': False,
        'duration': None,
    },
})
LOT_TYPE_CHOICES = [
    (key, value['name'])
    for key, value in LOT_SETTINGS.items()
]


class LOT(models.Model):
    uuid = models.CharField(_('UUID'), max_length=50)
    type = models.SlugField(_('LOT type'), max_length=50,
                            choices=LOT_TYPE_CHOICES)
    user = models.ForeignKey(User, verbose_name=_('user'))
    session_data = models.TextField(_('Jsoned Session Data'), blank=True)
    created = models.DateTimeField(_('Creation date'), auto_now_add=True)
    next_url = models.URLField(blank=True)

    def verify(self):
        if self.type not in LOT_SETTINGS:
            return False

        verify_setting = LOT_SETTINGS[self.type]

        duration = verify_setting.get('duration', None)
        verify_func = verify_setting.get('verify-func', lambda x: True)

        if not verify_func(self):
            return False

        if duration is None:
            return True

        return (now() - self.created).total_seconds() < duration

    def delete_on_fail(self):
        if self.type not in LOT_SETTINGS:
            return True

        return LOT_SETTINGS[self.type].get('delete-on-fail', True)

    def is_one_time(self):
        return LOT_SETTINGS.get(self.type, {}).get('one-time', False)

    def save(self, *args, **kwargs):
        if self.id and not kwargs.pop('force_modification', False):
            raise Exception('Modification not allowed without '
                            'force_modification parameter on save.')
        self.uuid = uuid4()
        super(LOT, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"{0} ({1})".format(self.get_type_display(), self.uuid)



import json

from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.shortcuts import get_object_or_404, resolve_url
from django.utils.http import is_safe_url
from django.views.generic import View

from django.contrib.auth import authenticate, login

from .models import LOT


class LOTLogin(View):
    def get(self, request, uuid):
        lot = get_object_or_404(LOT, uuid=uuid)
        if not lot.verify():
            lot.delete()
            return HttpResponseNotFound()

        user = authenticate(lot_uuid=uuid)
        login(request, user)

        try:
            session_data = json.loads(lot.session_data)
            request.session.update(session_data)
        except Exception:
            # If not correctly serialized not set the session_data
            pass

        if lot.is_one_time():
            lot.delete()

        redirect_to = request.GET.get('next')
        if lot.next_url:
            redirect_to = resolve_url(lot.next_url)

        if not is_safe_url(url=redirect_to, host=request.get_host()):
            redirect_to = resolve_url(settings.LOGIN_REDIRECT_URL)

        return HttpResponseRedirect(redirect_to)



import json
import re

from django.conf import settings
from django.contrib.auth import authenticate, login

from .models import LOT

uuidRegex = re.compile("^[\da-f]{8}-([\da-f]{4}-){3}[\da-f]{12}$")


class LOTMiddleware(object):
    def process_request(self, request):
        lot_uuid = request.GET.get(settings.LOT_MIDDLEWARE_PARAM_NAME, None)
        if lot_uuid:
            if not uuidRegex.match(lot_uuid):
                return None

            try:
                lot = LOT.objects.get(uuid=lot_uuid)
            except LOT.DoesNotExist:
                return None

            if not lot.verify():
                if lot.delete_on_fail():
                    lot.delete()
                return None

            user = authenticate(lot_uuid=lot_uuid)
            login(request, user)

            try:
                session_data = json.loads(lot.session_data)
                request.session.update(session_data)
            except Exception:
                # If not correctly serialized not set the session_data
                pass

            if lot.is_one_time():
                lot.delete()


class LOTAuthenticationMiddleware(object):
    '''Authenticate using a Header'''
    def process_request(self, request):
        try:
            token = request.META['HTTP_X_AUTH_TOKEN']
        except KeyError:
            return None

        if not uuidRegex.match(token):
            return None

        try:
            lot = LOT.objects.select_related('user').get(uuid=token)
        except LOT.DoesNotExist:
            return None

        if not lot.verify():
            if lot.delete_on_fail():
                lot.delete()
            return None

        request.user = lot.user

        try:
            session_data = json.loads(lot.session_data)
            request.session.update(session_data)
        except:
            pass

        if lot.is_one_time():
            lot.delete()



from django.contrib.auth.backends import ModelBackend

from .models import LOT


class LOTBackend(ModelBackend):
    def authenticate(self, lot_uuid=None, **kwargs):
        try:
            lot = LOT.objects.get(uuid=lot_uuid)
        except LOT.DoesNotExist:
            return None
        return lot.user



from django.contrib import admin
from .models import LOT


class LOTAdmin(admin.ModelAdmin):
    list_display = ('uuid', 'user', 'type', 'created',)
    list_filter = ('type',)
    readonly_fields = ('uuid',)

admin.site.register(LOT, LOTAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='LOT',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', models.CharField(verbose_name='UUID', max_length=50)),
                ('type', models.SlugField(choices=[('fast-login', 'Fast login'), ('slow-login', 'Slow login'), ('always-login', 'Always login')], verbose_name='LOT type')),
                ('session_data', models.TextField(blank=True, verbose_name='Jsoned Session Data')),
                ('created', models.DateTimeField(verbose_name='Creation date', auto_now_add=True)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup
import os

os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'

description = """
Django Login over Token.
"""

setup(
    name = "django-lot",
    url = "https://github.com/jespino/django-lot",
    author = "Jesús Espino, Curtis Malone",
    author_email = "jespinog@gmail.com, curtis@tinbrain.net",
    version=':versiontools:lot:',
    packages = [
        "lot",
        "lot.migrations",
    ],
    description = description.strip(),
    install_requires=['django >= 1.3.0', 'simplejson >= 3.3.0'],
    setup_requires = [
        'versiontools >= 1.8',
    ],
    zip_safe=False,
    include_package_data = False,
    package_data = {},
    test_suite = 'nose.collector',
    tests_require = ['nose >= 1.2.1', 'django >= 1.3.0'],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
        "Environment :: Web Environment",
        "Framework :: Django",
    ],
)
